﻿using System;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace Post11592
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private PolygonShape polygon;

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;

            InMemoryFeatureLayer polygonLayer = new InMemoryFeatureLayer();
            polygonLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.State2;
            polygonLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            polygon = (PolygonShape)BaseShape.CreateShapeFromWellKnownData("POLYGON((30.234375 61.875,106.875 60.46875,110.390625 16.5234375,103.7109375 -27.0703125,57.65625 -17.578125,11.25 -17.578125,-14.0625 -4.5703125,-29.53125 17.578125,-37.96875 46.0546875,-7.03125 47.8125,-1.7578125 57.3046875,10.1953125 65.390625,17.578125 66.09375,30.234375 61.875))");
            Feature feature = new Feature(polygon);
            polygonLayer.InternalFeatures.Add(feature);

            LayerOverlay overlay = new LayerOverlay();
            overlay.Layers.Add(polygonLayer);

            polygonLayer.Open();
            wpfMap1.CurrentExtent = polygonLayer.GetBoundingBox();
            polygonLayer.Close();

            wpfMap1.Overlays.Add(overlay);
            wpfMap1.Refresh();
        }

        private void btnTrack_Click(object sender, RoutedEventArgs e)
        {
            if (wpfMap1.TrackOverlay.TrackMode == TrackMode.None)
            {
                wpfMap1.TrackOverlay.TrackMode = TrackMode.Line;
                wpfMap1.TrackOverlay.MouseMoved += new EventHandler<MouseMovedTrackInteractiveOverlayEventArgs>(TrackOverlay_MouseMoved);
            }
            else
            {
                wpfMap1.TrackOverlay.TrackMode = TrackMode.None;
            }
        }

        void TrackOverlay_MouseMoved(object sender, MouseMovedTrackInteractiveOverlayEventArgs e)
        {
            if (wpfMap1.TrackOverlay.TrackMode == TrackMode.Line)
            {
                double distance = double.MaxValue;
                PointShape nearestPoint = new PointShape();
                for (int i = 0; i < polygon.OuterRing.Vertices.Count - 1; i++)
                {
                    Vertex[] line = new Vertex[] { polygon.OuterRing.Vertices[i], polygon.OuterRing.Vertices[i + 1] };
                    double minx = polygon.OuterRing.Vertices[i].X > polygon.OuterRing.Vertices[i + 1].X ? polygon.OuterRing.Vertices[i + 1].X : polygon.OuterRing.Vertices[i].X;
                    double maxx = polygon.OuterRing.Vertices[i].X < polygon.OuterRing.Vertices[i + 1].X ? polygon.OuterRing.Vertices[i + 1].X : polygon.OuterRing.Vertices[i].X;
                    double miny = polygon.OuterRing.Vertices[i].Y > polygon.OuterRing.Vertices[i + 1].Y ? polygon.OuterRing.Vertices[i + 1].Y : polygon.OuterRing.Vertices[i].Y;
                    double maxy = polygon.OuterRing.Vertices[i].Y < polygon.OuterRing.Vertices[i + 1].Y ? polygon.OuterRing.Vertices[i + 1].Y : polygon.OuterRing.Vertices[i].Y;

                    if (((minx < e.MovedVertex.X) && (maxx > e.MovedVertex.X)) || ((miny < e.MovedVertex.Y) && (maxy > e.MovedVertex.Y)))
                    {
                        LineShape xLine = new LineShape(line);
                        double newDistance = xLine.GetDistanceTo(new PointShape(e.MovedVertex), GeographyUnit.DecimalDegree, DistanceUnit.Meter);
                        if (distance > newDistance)
                        {
                            distance = newDistance;
                            nearestPoint = xLine.GetClosestPointTo(new PointShape(e.MovedVertex), GeographyUnit.DecimalDegree);
                        }
                    }
                }
                if (distance < 600000)
                {
                    e.MovedVertex = new Vertex(nearestPoint.X, nearestPoint.Y);
                }
            }
        }

    }
}
